using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.IO;
using System.Net.Mime;

using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.TreeEngine;
using CMS.LicenseProvider;
using CMS.SettingsProvider;
using CMS.SiteProvider;
using CMS.EmailEngine;
using CMS.UIControls;

public partial class CMSSiteManager_Support_SubmitIssue : SiteManagerPage
{
    #region "Variables"

    private string mSetts = "";
    private bool globalLoaded = false;

    #endregion


    protected void Page_Load(object sender, EventArgs e)
    {
        // Setup page title text and image
        this.CurrentMaster.Title.TitleText = ResHelper.GetString("SubmitIssue.Title");
        this.CurrentMaster.Title.TitleImage = GetImageUrl("CMSModules/CMS_Support/SubmitIssueLarge.png");

        this.CurrentMaster.Title.HelpTopicName = "submit_issue";
        this.CurrentMaster.Title.HelpName = "helpTopic";

        lblEmail.Text = ResHelper.GetString("Support.SubmiIssue.Email");
        lblSysInfo.Text = ResHelper.GetString("Support.SubmiIssue.SysInfo");
        lblTemplate.Text = ResHelper.GetString("Support.SubmiIssue.Template");
        lblAttachment.Text = ResHelper.GetString("Support.SubmiIssue.Attachment");
        lblHtmlIssue.Text = ResHelper.GetString("Support.SubmiIssue.FckIssuie");

        btnSend.Text = ResHelper.GetString("Support.SubmiIssue.Send");

        radAspx.Text = ResHelper.GetString("Support.SubmiIssue.Aspx");
        radMix.Text = ResHelper.GetString("Support.SubmiIssue.Mix");
        radPortal.Text = ResHelper.GetString("Support.SubmiIssue.Portal");
        radDontKnow.Text = ResHelper.GetString("Support.SubmiIssue.DontKnow");

        rfvEmail.Text = ResHelper.GetString("Support.SubmiIssue.EmailEmpty");
        rfvSubject.Text = ResHelper.GetString("Support.SubmiIssue.SubjectEmpty");

        lblSettings.Text = ResHelper.GetString("Support.SubmiIssue.Settings");

        if (!RequestHelper.IsPostBack())
        {
            htmlTemplateBody.AutoDetectLanguage = false;
            htmlTemplateBody.DefaultLanguage = System.Threading.Thread.CurrentThread.CurrentCulture.TwoLetterISOLanguageName;
            htmlTemplateBody.EditorAreaCSS = "";
            htmlTemplateBody.ToolbarSet = "Basic";

            txtSysInfo.Text = GetSystemInformation();
        }

        if (RequestHelper.IsPostBack())
        {
            GetSettingsInfo();
        }
    }


    /// <summary>
    /// Clear form
    /// </summary>
    public void ClearForm()
    {
        txtEmail.Text = "";
        txtSubject.Text = "";
        txtSysInfo.Text = GetSystemInformation();

        htmlTemplateBody.ResolvedValue = "";
        chkSettings.Checked = true;
        radDontKnow.Checked = true;
    }


    /// <summary>
    /// Return settings 
    /// </summary>
    public void GetSettingsInfo()
    {
        string setsText = "----------Settings---------\r\n\r\n";
        string prefix = "";

        DataSet ds = SiteInfoProvider.GetSites(null, null, "SiteID, SiteName");
        if (!DataHelper.DataSourceIsEmpty(ds))
        {
            foreach (DataRow dr in ds.Tables[0].Rows)
            {                
                int siteId = ValidationHelper.GetInteger(dr["SiteId"], 0);
                string siteName = ValidationHelper.GetString(dr["SiteName"], String.Empty);

                if ((siteId > 0) && !String.IsNullOrEmpty(siteName))
                {
                    prefix = " - ";

                    DataSet setsDs = SettingsKeyProvider.GetSettingsCategories(siteId);

                    if (!globalLoaded)
                    {
                        try
                        {
                            globalLoaded = true;
                            setsText += prefix + "GLOBAL SETTINGS" + "\r\n";
                            foreach (DataRow setsDr in setsDs.Tables[0].Rows)
                            {
                                prefix = "\t - ";
                                setsText = setsText + prefix + ValidationHelper.GetString(setsDr["CategoryDisplayName"], "") + "\r\n";

                                DataSet keyDs = CMS.SettingsProvider.SettingsKeyProvider.GetSettingsKeys(0, ValidationHelper.GetInteger(setsDr["CategoryId"], 0));
                                if (!DataHelper.DataSourceIsEmpty(keyDs))
                                {
                                    prefix = "\t\t - ";

                                    foreach (DataRow keyDr in keyDs.Tables[0].Rows)
                                    {
                                        setsText = setsText + prefix + ValidationHelper.GetString(keyDr["KeyName"], "") + " '" + ValidationHelper.GetString(keyDr["KeyValue"], "") + "'(" + ValidationHelper.GetString(keyDr["KeyType"], "") + ")" + "\r\n";
                                    }
                                }
                            }

                            setsText += "\r\n";
                        }
                        catch
                        {
                            globalLoaded = true;
                        }
                    }

                    if (!DataHelper.DataSourceIsEmpty(setsDs))
                    {
                        prefix = " - ";

                        setsText += prefix + siteName + "\r\n";
                        foreach (DataRow setsDr in setsDs.Tables[0].Rows)
                        {
                            prefix = "\t - ";
                            setsText = setsText + prefix + ValidationHelper.GetString(setsDr["CategoryDisplayName"], "") + "\r\n";

                            DataSet keyDs = CMS.SettingsProvider.SettingsKeyProvider.GetSettingsKeys(siteId, ValidationHelper.GetInteger(setsDr["CategoryId"], 0));
                            if (!DataHelper.DataSourceIsEmpty(keyDs))
                            {
                                prefix = "\t\t - ";

                                foreach (DataRow keyDr in keyDs.Tables[0].Rows)
                                {
                                    setsText = setsText + prefix + ValidationHelper.GetString(keyDr["KeyName"], "") + " '" + ValidationHelper.GetString(keyDr["KeyValue"], "") + "'(" + ValidationHelper.GetString(keyDr["KeyType"], "") + ")" + "\r\n";
                                }
                            }
                        }
                    }

                    setsText += "\r\n";
                }
            }
        }

        mSetts += setsText;
    }


    /// <summary>
    /// Return system informations
    /// </summary>
    public string GetSystemInformation()
    {
        string toReturn = "";

        toReturn = "CMS version : " + CMSContext.SYSTEM_VERSION + " Build : " + typeof(TreeProvider).Assembly.GetName().Version.Major + "." + typeof(TreeProvider).Assembly.GetName().Version.Minor + "." + typeof(TreeProvider).Assembly.GetName().Version.Build + "\n";

        toReturn += "OS version : " + System.Environment.OSVersion + "\n";

        LicenseKeyInfo lki = null;

        if (CMSContext.CurrentSite != null)
        {
            lki = LicenseKeyInfoProvider.GetLicenseKeyInfo(CMSContext.CurrentSite.DomainName);
        }

        if (lki != null)
        {
            toReturn += "License info : " + lki.Domain + ", " + lki.Edition + ", " + lki.ExpirationDateReal.ToString(DateTimeHelper.DefaultIFormatProvider) + ", " + lki.Version;
            
            string packages = lki.GetValue("LicensePackages") as string;
            if (!String.IsNullOrEmpty(packages))
            {
                toReturn += ", " + packages;
            }
            
        }

        return toReturn;
    }


    /// <summary>
    /// Send e-mail
    /// </summary>
    protected void btnSend_Click(object sender, EventArgs e)
    {
        string result = new Validator().NotEmpty(txtSubject.Text, ResHelper.GetString("Support.SubmiIssue.SubjectEmpty")).NotEmpty(txtEmail.Text, ResHelper.GetString("Support.SubmiIssue.EmailEmpty")).IsRegularExp(txtEmail.Text, @"^([\w0-9_-]+(\.[\w0-9_-]+)*@[\w0-9_-]+(\.[\w0-9_-]+)+)*$", ResHelper.GetString("Support.SubmiIssue.EmailFormat")).Result;

        if (result == "")
        {
            EmailMessage message = new EmailMessage();
            message.EmailFormat = EmailFormatEnum.Html;
            message.From = txtEmail.Text;
            message.Subject = txtSubject.Text;
            message.Recipients = "support@kentico.com";

            message.Body += "<html><head></head><body>";

            message.Body = htmlTemplateBody.ResolvedValue;

            message.Body += "<br /><div>=============== System information ==================<br />";
            message.Body += txtSysInfo.Text.Replace("\r\n", "<br />");
            message.Body += "<br />==============================================</div><br />";

            message.Body += "<div>================ Template type ==================<br />";
            if (radAspx.Checked) { message.Body += "ASPX Templates<br />"; }
            if (radPortal.Checked) { message.Body += "Portal engine<br />"; }
            if (radMix.Checked) { message.Body += "Both<br />"; }
            if (radDontKnow.Checked) { message.Body += "I don't know<br />"; }
            message.Body += "=============================================</div>";

            message.Body += "</body></html>";

            // Add settings attachment
            if (chkSettings.Checked)
            {
                Stream stream = new MemoryStream();

                // Put the file content on the stream.
                StreamWriter writer = new StreamWriter(stream, System.Text.UnicodeEncoding.Unicode);
                writer.Write(mSetts);
                writer.Flush();
                stream.Seek(0, SeekOrigin.Begin);

                message.Attachments.Add(new System.Net.Mail.Attachment(stream, "settings.txt", MediaTypeNames.Application.Octet));
            }

            if (fileUpload.HasFile)
            {
                System.Net.Mail.Attachment at = new System.Net.Mail.Attachment(fileUpload.PostedFile.InputStream, fileUpload.FileName, MediaTypeNames.Application.Octet);

                if (at != null)
                {
                    message.Attachments.Add(at); ;
                }
            }
            try
            {
                EmailSender.SendEmail(null, message, true);

                lblInfo.Visible = true;
                lblInfo.Text = ResHelper.GetString("Support.Success");

                ClearForm();
            }
            catch
            {
                lblError.Visible = true;
                lblError.Text = ResHelper.GetString("Support.Failed"); ;
            }
        }
        else
        {
            lblError.Visible = true;
            lblError.Text = result;
        }
    }
}