using System;
using System.Data;
using System.Web;
using System.Web.UI;
using System.Web.Caching;

using CMS.PortalControls;
using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.Controls;

public partial class CMSWebParts_Membership_OnlineUsers : CMSAbstractWebPart
{
    #region "Public properties"

    /// <summary>
    /// Select only top n users.
    /// </summary>
    public int SelectTopN
    {
        get
        {
            return ValidationHelper.GetInteger(GetValue("SelectTopN"), 0);
        }
        set
        {
            SetValue("SelectTopN", value);
        }
    }


    /// <summary>
    /// Select only users localized in specified path.  
    /// </summary>
    public string Path
    {
        get
        {
            return ValidationHelper.GetString(GetValue("Path"), "");
        }
        set
        {
            SetValue("Path", value);
        }
    }


    /// <summary>
    /// Gets or sets the transformation name
    /// </summary>
    public string TransformationName
    {
        get
        {
            return ValidationHelper.GetString(GetValue("TransformationName"), "");
        }
        set
        {
            SetValue("TransformationName", value);
        }
    }


    /// <summary>
    /// Gets or sets the additional info text
    /// </summary>
    public string AdditionalInfoText
    {
        get
        {
            return ValidationHelper.GetString(GetValue("AdditionalInfoText"), "");
        }
        set
        {
            SetValue("AdditionalInfoText", value);
        }
    }


    /// <summary>
    /// Gets or sets the no users on-line text
    /// </summary>
    public string NoUsersOnlineText
    {
        get
        {
            return ValidationHelper.GetString(GetValue("NoUsersOnlineText"), "");
        }
        set
        {
            SetValue("NoUsersOnlineText", value);
        }
    }


    /// <summary>
    /// Gets or sets columns to be retrieved from database
    /// </summary>
    public string Columns
    {
        get
        {
            return ValidationHelper.GetString(GetValue("Columns"), "");
        }
        set
        {
            SetValue("Columns", value);
        }
    }

    #endregion


    #region "Events"

    /// <summary>
    /// OnPreRender override method.
    /// </summary>
    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);
        SetupControl();
    }

    #endregion


    #region "Methods"

    /// <summary>
    /// Setups control properties.
    /// </summary>
    protected void SetupControl()
    {
        // Check StopProcessing property
        if (StopProcessing)
        {
            Visible = false;
        }
        else
        {
            SetContext();

            DataSet users = null;
            bool transLoaded = false;

            // Load transformation
            if (!string.IsNullOrEmpty(TransformationName))
            {
                repUsers.ItemTemplate = CMSDataProperties.LoadTransformation(this, TransformationName, false);
                transLoaded = true;
            }

            if ((transLoaded) || (!String.IsNullOrEmpty(Path)))
            {
                // Get or generate cache item name
                string useCacheItemName = CacheHelper.GetCacheItemName(this.CacheItemName, "onlineusers", CMSContext.CurrentSiteName, SelectTopN, Columns, Path);

                // Try to get data from cache 
                if ((this.CacheMinutes > 0) && CacheHelper.TryGetItem<DataSet>(useCacheItemName, out users))
                {
                    // Retrieved from cache
                }
                else
                {
                    // Get the data
                    users = SessionManager.GetOnlineUsers(null, null, SelectTopN, Path, CMSContext.CurrentSiteName, false, false);

                    // Prepare the cache dependency
                    if (this.CacheMinutes > 0)
                    {
                        CacheDependency cd = GetCacheDependency();
                        CacheHelper.Add(useCacheItemName, users, cd, DateTime.Now.AddMinutes(CacheMinutes), System.Web.Caching.Cache.NoSlidingExpiration);
                    }
                }

                // Data bind
                if (!DataHelper.DataSourceIsEmpty(users))
                {
                    // Set to repeater
                    repUsers.DataSource = users;
                    repUsers.DataBind();
                }
            }

            int authenticated = 0;
            int publicUsers = 0;

            string numbers = string.Empty;

            // Get or generate cache item name
            string cacheItemNameNumbers = this.CacheItemName;
            if (!string.IsNullOrEmpty(cacheItemNameNumbers))
            {
                cacheItemNameNumbers += "Number";
            }

            // Get or generate cache item name
            string useCacheItemNameNumber = CacheHelper.GetCacheItemName(cacheItemNameNumbers, "onlineusersnumber", CMSContext.CurrentSiteName, Path);

            // No caching
            if ((this.CacheMinutes > 0) && CacheHelper.TryGetItem<string>(useCacheItemNameNumber, out numbers))
            {
                // Retrieved from cache
                string[] nums = numbers.Split(';');

                publicUsers = ValidationHelper.GetInteger(nums[0], 0);
                authenticated = ValidationHelper.GetInteger(nums[1], 0);
            }
            else
            {
                // Get the data
                SessionManager.GetUsersNumber(CurrentSiteName, Path, false, false, out publicUsers, out authenticated);

                // Save to the cache
                if (this.CacheMinutes > 0)
                {
                    CacheDependency cd = GetCacheDependency();
                    CacheHelper.Add(useCacheItemNameNumber, publicUsers.ToString() + ";" + authenticated.ToString(), cd, DateTime.Now.AddMinutes(CacheMinutes), System.Web.Caching.Cache.NoSlidingExpiration);
                }
            }

            // Check if at least one user is online
            if ((publicUsers + authenticated) == 0)
            {
                ltrAdditionaInfos.Text = NoUsersOnlineText;
            }
            else
            {
                ltrAdditionaInfos.Text = string.Format(AdditionalInfoText, publicUsers + authenticated, publicUsers, authenticated);
            }

        }

        ReleaseContext();
    }


    /// <summary>
    /// Clears the cached items
    /// </summary>
    public override void ClearCache()
    {
        string useCacheItemName = DataHelper.GetNotEmpty(CacheItemName, CMSContext.BaseCacheKey + "|" + Context.Request.Url + "|" + ClientID + "|" + "onlineUsersWebPart");

        CacheHelper.ClearCache(useCacheItemName);

        // Get or generate cache item name for number
        string cacheItemNameNumbers = CacheItemName;
        if (!string.IsNullOrEmpty(cacheItemNameNumbers))
        {
            cacheItemNameNumbers += "Number";
        }

        string useCacheItemNameNumber = DataHelper.GetNotEmpty(cacheItemNameNumbers, CMSContext.BaseCacheKey + "|" + Context.Request.Url + "|" + ClientID + "|" + "onlineUsersWebPartNumber");

        CacheHelper.ClearCache(useCacheItemNameNumber);
    }

    #endregion
}
